@@ -1024,7 +1024,7 @@ static struct fuse *req_fuse_prepare(fuse_req_t req)
     return c->ctx.fuse;
 }
 
-static inline void reply_err(fuse_req_t req, int err)
+static void reply_err(fuse_req_t req, int err)
 {
     /* fuse_reply_err() uses non-negated errno values */
     fuse_reply_err(req, -err);
