@@ -3,7 +3,9 @@
  *
  * Copyright (c) 2005-2007 Yura Pakhuchiy
  * Copyright (c) 2005 Yuval Fledel
- * Copyright (c) 2006-2008 Szabolcs Szakacsits
+ * Copyright (c) 2006-2009 Szabolcs Szakacsits
+ * Copyright (c) 2007-2009 Jean-Pierre Andre
+ * Copyright (c) 2009 Erik Larsson
  *
  * This file is originated from the Linux-NTFS project.
  *
@@ -89,14 +91,9 @@
 #include "unistr.h"
 #include "layout.h"
 #include "index.h"
-#include "utils.h"
 #include "ntfstime.h"
 #include "misc.h"
 
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
-
 typedef enum {
 	FSTYPE_NONE,
 	FSTYPE_UNKNOWN,
@@ -132,7 +129,7 @@ typedef struct {
 	BOOL ro;
 	BOOL show_sys_files;
 	BOOL silent;
-	BOOL force;
+	BOOL recover;
 	BOOL hiberfile;
 	BOOL debug;
 	BOOL no_detach;
@@ -152,25 +149,22 @@ static char def_opts[] = "silent,allow_other,nonempty,";
 static ntfs_fuse_context_t *ctx;
 static u32 ntfs_sequence;
 
-static const char *locale_msg =
-"WARNING: Couldn't set locale to '%s' thus some file names may not\n"
-"         be correct or visible. Please see the potential solution at\n"
-"         http://ntfs-3g.org/support.html#locale\n";
-
 static const char *usage_msg = 
 "\n"
 "%s %s %s %d - Third Generation NTFS Driver\n"
 "\n"
-"Copyright (C) 2006-2008 Szabolcs Szakacsits\n"
 "Copyright (C) 2005-2007 Yura Pakhuchiy\n"
+"Copyright (C) 2006-2009 Szabolcs Szakacsits\n"
+"Copyright (C) 2007-2009 Jean-Pierre Andre\n"
+"Copyright (C) 2009 Erik Larsson\n"
 "\n"
 "Usage:    %s [-o option[,...]] <device|image_file> <mount_point>\n"
 "\n"
-"Options:  ro (read-only mount), force, remove_hiberfile, locale=,\n" 
-"          uid=, gid=, umask=, fmask=, dmask=, streams_interface=.\n"
-"          Please see the details in the manual.\n"
+"Options:  ro (read-only mount), remove_hiberfile, uid=, gid=,\n" 
+"          umask=, fmask=, dmask=, streams_interface=.\n"
+"          Please see the details in the manual (type: man ntfs-3g).\n"
 "\n"
-"Examples: ntfs-3g -o force /dev/sda1 /mnt/windows\n"
+"Example: ntfs-3g /dev/sda1 /mnt/windows\n"
 "\n"
 "%s";
 
@@ -261,7 +255,10 @@ static int ntfs_fuse_statfs(const char *path __attribute__((unused)),
 	if (!vol)
 		return -ENODEV;
 	
-	/* File system block size, used for optimal transfer block size. */
+	/* 
+	 * File system block size. Used to calculate used/free space by df.
+	 * Incorrectly documented as "optimal transfer block size". 
+	 */
 	sfs->f_bsize = vol->cluster_size;
 	
 	/* Fundamental file system block size, used as the unit. */
@@ -345,6 +342,87 @@ static void set_fuse_error(int *err)
 		*err = -errno;
 }
 
+#if defined(__APPLE__) || defined(__DARWIN__)
+static void *ntfs_macfuse_init(struct fuse_conn_info *conn)
+{
+	FUSE_ENABLE_XTIMES(conn);
+	return NULL;
+}
+
+static int ntfs_macfuse_getxtimes(const char *org_path,
+		struct timespec *bkuptime, struct timespec *crtime)
+{
+	int res = 0;
+	ntfs_inode *ni;
+	char *path = NULL;
+	ntfschar *stream_name;
+	int stream_name_len;
+
+	stream_name_len = ntfs_fuse_parse_path(org_path, &path, &stream_name);
+	if (stream_name_len < 0)
+		return stream_name_len;
+	memset(bkuptime, 0, sizeof(struct timespec));
+	memset(crtime, 0, sizeof(struct timespec));
+	ni = ntfs_pathname_to_inode(ctx->vol, NULL, path);
+	if (!ni) {
+		res = -errno;
+		goto exit;
+	}
+	
+	/* We have no backup timestamp in NTFS. */
+	crtime->tv_sec = ni->creation_time;
+exit:
+	if (ntfs_inode_close(ni))
+		set_fuse_error(&res);
+	free(path);
+	if (stream_name_len)
+		free(stream_name);
+	return res;
+}
+
+int ntfs_macfuse_setcrtime(const char *path, const struct timespec *tv)
+{
+	ntfs_inode *ni;
+	int res = 0;
+
+	if (ntfs_fuse_is_named_data_stream(path))
+		return -EINVAL; /* n/a for named data streams. */
+	ni = ntfs_pathname_to_inode(ctx->vol, NULL, path);
+	if (!ni)
+		return -errno;
+	
+	if (tv) {
+		ni->creation_time = tv->tv_sec;
+		ntfs_fuse_update_times(ni, NTFS_UPDATE_CTIME);
+	}
+
+	if (ntfs_inode_close(ni))
+		set_fuse_error(&res);
+	return res;
+}
+
+int ntfs_macfuse_setbkuptime(const char *path, const struct timespec *tv)
+{
+	ntfs_inode *ni;
+	int res = 0;
+	
+	if (ntfs_fuse_is_named_data_stream(path))
+		return -EINVAL; /* n/a for named data streams. */
+	ni = ntfs_pathname_to_inode(ctx->vol, NULL, path);
+	if (!ni)
+		return -errno;
+	
+	/* 
+	 * Doing nothing while pretending to do something. NTFS has no backup
+	 * time. If this function is not implemented then some apps break. 
+	 */
+	
+	if (ntfs_inode_close(ni))
+		set_fuse_error(&res);
+	return res;
+}
+#endif /* defined(__APPLE__) || defined(__DARWIN__) */
+
 static int ntfs_fuse_getattr(const char *org_path, struct stat *stbuf)
 {
 	int res = 0;
@@ -558,9 +636,9 @@ static int ntfs_fuse_filler(ntfs_fuse_fill_context_t *fill_ctx,
 		return 0;
 	
 	if (ntfs_ucstombs(name, name_len, &filename, 0) < 0) {
-		ntfs_log_perror("Skipping unrepresentable filename (inode %llu)",
+		ntfs_log_perror("Filename decoding failed (inode %llu)",
 				(unsigned long long)MREF(mref));
-		return 0;
+		return -1;
 	}
 	
 	if (ntfs_fuse_is_named_data_stream(filename)) {
@@ -1472,7 +1550,7 @@ static int ntfs_fuse_setxattr(const char *path, const char *name,
 		return -EOPNOTSUPP;
 	if (strncmp(name, nf_ns_xattr_preffix, nf_ns_xattr_preffix_len) ||
 			strlen(name) == (size_t)nf_ns_xattr_preffix_len)
-		return -EACCES;
+		return -EOPNOTSUPP;
 	ni = ntfs_pathname_to_inode(ctx->vol, NULL, path);
 	if (!ni)
 		return -errno;
@@ -1500,6 +1578,11 @@ static int ntfs_fuse_setxattr(const char *path, const char *name,
 			res = -errno;
 			goto exit;
 		}
+	} else {
+		if (ntfs_attr_truncate(na, (s64)size)) {
+			res = -errno;
+			goto exit;
+		}
 	}
 	res = ntfs_attr_pwrite(na, 0, size, value);
 	if (res != (s64) size)
@@ -1601,6 +1684,13 @@ static struct fuse_operations ntfs_3g_ops = {
 	.removexattr	= ntfs_fuse_removexattr,
 	.listxattr	= ntfs_fuse_listxattr,
 #endif /* HAVE_SETXATTR */
+#if defined(__APPLE__) || defined(__DARWIN__)
+	.init		= ntfs_macfuse_init,
+	/* MacFUSE extensions. */
+	.getxtimes	= ntfs_macfuse_getxtimes,
+	.setcrtime	= ntfs_macfuse_setcrtime,
+	.setbkuptime	= ntfs_macfuse_setbkuptime
+#endif /* defined(__APPLE__) || defined(__DARWIN__) */
 };
 
 static int ntfs_fuse_init(void)
@@ -1610,9 +1700,16 @@ static int ntfs_fuse_init(void)
 		return -1;
 	
 	*ctx = (ntfs_fuse_context_t) {
-		.uid = getuid(),
-		.gid = getgid(),
+		.uid     = getuid(),
+		.gid     = getgid(),
+#if defined(linux)			
+		.streams = NF_STREAMS_INTERFACE_XATTR,
+#else			
 		.streams = NF_STREAMS_INTERFACE_NONE,
+#endif			
+		.atime   = ATIME_RELATIVE,
+		.silent  = TRUE,
+		.recover = TRUE
 	};
 	return 0;
 }
@@ -1625,8 +1722,8 @@ static int ntfs_open(const char *device)
 		flags |= MS_EXCLUSIVE;
 	if (ctx->ro)
 		flags |= MS_RDONLY;
-	if (ctx->force)
-		flags |= MS_FORCE;
+	if (ctx->recover)
+		flags |= MS_RECOVER;
 	if (ctx->hiberfile)
 		flags |= MS_IGNORE_HIBERFILE;
 
@@ -1726,9 +1823,6 @@ static char *parse_mount_options(const char *orig_opts)
 		return NULL;
 	}
 	
-	ctx->silent = TRUE;
-	ctx->atime  = ATIME_RELATIVE;
-	
 	s = options;
 	while (s && *s && (val = strsep(&s, ","))) {
 		opt = strsep(&val, "=");
@@ -1804,10 +1898,14 @@ static char *parse_mount_options(const char *orig_opts)
 			if (bogus_option_value(val, "silent"))
 				goto err_exit;
 			ctx->silent = TRUE;
-		} else if (!strcmp(opt, "force")) {
-			if (bogus_option_value(val, "force"))
+		} else if (!strcmp(opt, "recover")) {
+			if (bogus_option_value(val, "recover"))
 				goto err_exit;
-			ctx->force = TRUE;
+			ctx->recover = TRUE;
+		} else if (!strcmp(opt, "norecover")) {
+			if (bogus_option_value(val, "norecover"))
+				goto err_exit;
+			ctx->recover = FALSE;
 		} else if (!strcmp(opt, "remove_hiberfile")) {
 			if (bogus_option_value(val, "remove_hiberfile"))
 				goto err_exit;
@@ -1815,8 +1913,7 @@ static char *parse_mount_options(const char *orig_opts)
 		} else if (!strcmp(opt, "locale")) {
 			if (missing_option_value(val, "locale"))
 				goto err_exit;
-			if (!setlocale(LC_ALL, val))
-				ntfs_log_error(locale_msg, val);
+			setlocale(LC_ALL, val);
 		} else if (!strcmp(opt, "streams_interface")) {
 			if (missing_option_value(val, "streams_interface"))
 				goto err_exit;
@@ -1915,11 +2012,12 @@ static int parse_options(int argc, char *argv[])
 {
 	int c;
 
-	static const char *sopt = "-o:hv";
+	static const char *sopt = "-o:hvV";
 	static const struct option lopt[] = {
 		{ "options",	 required_argument,	NULL, 'o' },
 		{ "help",	 no_argument,		NULL, 'h' },
 		{ "verbose",	 no_argument,		NULL, 'v' },
+		{ "version",	 no_argument,		NULL, 'V' },
 		{ NULL,		 0,			NULL,  0  }
 	};
 
@@ -1966,6 +2064,10 @@ static int parse_options(int argc, char *argv[])
 			 * we don't use it because mount(8) passes it.
 			 */
 			break;
+		case 'V':
+			ntfs_log_info("%s %s %s %d\n", EXEC_NAME, VERSION, 
+				      FUSE_TYPE, fuse_version());
+			exit(0);
 		default:
 			ntfs_log_error("%s: Unknown option '%s'.\n", EXEC_NAME,
 				       argv[optind - 1]);
@@ -2219,12 +2321,11 @@ int main(int argc, char *argv[])
 	if (drop_privs())
 		return NTFS_VOLUME_NO_PRIVILEGE;
 	
-	utils_set_locale();
+	ntfs_set_locale();
 	ntfs_log_set_handler(ntfs_log_handler_stderr);
 
 	if (parse_options(argc, argv)) {
-		ntfs_log_error("Please type '%s --help' for more "
-			       "information.\n", argv[0]);
+		usage();
 		return NTFS_VOLUME_SYNTAX_ERROR;
 	}
 
@@ -2301,7 +2402,7 @@ int main(int argc, char *argv[])
 	fuse_unmount(opts.mnt_point, ctx->fc);
 	fuse_destroy(fh);
 err_out:
-	utils_mount_error(opts.device, opts.mnt_point, err);
+	ntfs_mount_error(opts.device, opts.mnt_point, err);
 err2:
 	ntfs_close();
 	free(ctx);
